@@ -46,6 +46,10 @@ namespace acommon {
     inline ParmString(const PosibErr<const char *> &);
     inline ParmString(const PosibErr<String> &);
 
+    void set(const char * str, unsigned int sz = UINT_MAX) {
+      str_ = str, size_ = sz;
+    }
+
     bool empty() const {
       return str_ == 0 || str_[0] == '\0';
     }
@@ -59,43 +63,42 @@ namespace acommon {
     const char * str () const {
       return str_;
     }
-  private:
+  public: // but don't use unless you have to
     const char * str_;
     mutable unsigned int size_;
   };
-#ifndef __SUNPRO_CC
-  inline bool operator== (ParmString s1, ParmString s2)
+
+  static inline bool operator== (ParmString s1, ParmString s2)
   {
     if (s1.str() == 0 || s2.str() == 0)
       return s1.str() == s2.str();
     return strcmp(s1,s2) == 0;
   }
-#endif
-  inline bool operator== (const char * s1, ParmString s2)
+  static inline bool operator== (const char * s1, ParmString s2)
   {
     if (s1 == 0 || s2.str() == 0)
       return s1 == s2.str();
     return strcmp(s1,s2) == 0;
   }
-  inline bool operator== (ParmString s1, const char * s2)
+  static inline bool operator== (ParmString s1, const char * s2)
   {
     if (s1.str() == 0 || s2 == 0)
       return s1.str() == s2;
     return strcmp(s1,s2) == 0;
   }
-  inline bool operator!= (ParmString s1, ParmString s2)
+  static inline bool operator!= (ParmString s1, ParmString s2)
   {
     if (s1.str() == 0 || s2.str() == 0)
       return s1.str() != s2.str();
     return strcmp(s1,s2) != 0;
   }
-  inline bool operator!= (const char * s1, ParmString s2)
+  static inline bool operator!= (const char * s1, ParmString s2)
   {
     if (s1 == 0 || s2.str() == 0)
       return s1 != s2.str();
     return strcmp(s1,s2) != 0;
   }
-  inline bool operator!= (ParmString s1, const char * s2)
+  static inline bool operator!= (ParmString s1, const char * s2)
   {
     if (s1.str() == 0 || s2 == 0)
       return s1.str() != s2;
