@@ -20,6 +20,7 @@
 #include "getdata.hpp"
 #include "parm_string.hpp"
 #include "check_list.hpp"
+#include "speller_impl.hpp"
 
 using namespace std;
 
@@ -61,8 +62,9 @@ void CheckList::reset()
 // First some base level utility routines
 char * mystrdup(const char * s);                   // duplicate string
 char * myrevstrdup(const char * s);                // duplicate reverse of string
-char * mystrsep(const char ** sptr, const char delim); // parse into tokens with char delimiter
-bool   isSubset(const char * s1, const char * s2); // is affix s1 is a "subset" affix s2
+static char * mystrsep(const char ** sptr, const char delim); // parse into tokens with char delimiter
+static bool   isSubset(const char * s1, const char * s2); // is affix s1 is a "subset" affix s2
+static bool   isRevSubset(const char * s1, const char * end_of_s2, int len);
 
 PosibErr<void> AffixMgr::setup(ParmString affpath)
 {
@@ -114,7 +116,7 @@ AffixMgr::~AffixMgr()
 PosibErr<void> AffixMgr::parse_file(const char * affpath)
 {
   // io buffers
-  String key, data;
+  FixedBuffer<> buf; DataPair d;
  
   // affix type
   char ft;
@@ -127,35 +129,36 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath)
   // step one is to parse the affix file building up the internal
   // affix data structures
 
+  // FIXME: Make sure that the encoding used for the affix file
+  //   is the same as the internal encoding used for the language
 
   // read in each line ignoring any that do not
   // start with a known line type indicator
 
-  while (getdata_pair(afflst,key,data)) {
-
+  while (getdata_pair(afflst,d,buf)) {
     ft = ' ';
 
     /* parse in the name of the character set used by the .dict and .aff */
 
-    if (key == "SET") 
-      encoding = data;
+    if (d.key == "SET") 
+      encoding = d.value;
 
     /* parse in the flag used by the controlled compound words */
-    else if (key == "COMPOUNDFLAG")
-      compound = data;
+    else if (d.key == "COMPOUNDFLAG")
+      compound = d.value;
 
     /* parse in the flag used by the controlled compound words */
-    else if (key == "COMPOUNDMIN")
-      cpdmin = atoi(data.c_str()); // FiXME
+    else if (d.key == "COMPOUNDMIN")
+      cpdmin = atoi(d.value); // FiXME
 
-    else if (key == "TRY" || key == "REP");
+    else if (d.key == "TRY" || d.key == "REP");
 
-    else if (key == "PFX" || key == "SFX")
-      ft = key[0];
+    else if (d.key == "PFX" || d.key == "SFX")
+      ft = d.key[0];
 
     // parse this affix: P - prefix, S - suffix
     if (ft != ' ')
-      RET_ON_ERR(parse_affix(data, ft, afflst));
+      RET_ON_ERR(parse_affix(d.value, ft, afflst));
 
   }
   afflst.close();
@@ -511,16 +514,14 @@ void AffixMgr::encodeit(AffEntry * ptr, char * cs)
 
 
 // check word for prefixes
-BasicWordInfo AffixMgr::prefix_check (LookupInfo linf, ParmString word, 
-                                      CheckInfo & ci, GuessInfo * gi) const
+bool AffixMgr::prefix_check (const LookupInfo & linf, ParmString word, 
+                             CheckInfo & ci, GuessInfo * gi) const
 {
-  BasicWordInfo wordinfo;
  
   // first handle the special case of 0 length prefixes
   PfxEntry * pe = (PfxEntry *) pStart[0];
   while (pe) {
-    wordinfo = pe->check(linf,word,ci,gi);
-    if (wordinfo) return wordinfo;
+    if (pe->check(linf,word,ci,gi)) return true;
     pe = pe->next;
   }
   
@@ -530,61 +531,51 @@ BasicWordInfo AffixMgr::prefix_check (LookupInfo linf, ParmString word,
 
   while (pptr) {
     if (isSubset(pptr->key(),word)) {
-      wordinfo = pptr->check(linf,word,ci,gi);
-      if (wordinfo) return wordinfo;
+      if (pptr->check(linf,word,ci,gi)) return true;
       pptr = pptr->next_eq;
     } else {
       pptr = pptr->next_ne;
     }
   }
     
-  return BasicWordInfo();
+  return false;
 }
 
 
 // check word for suffixes
-BasicWordInfo AffixMgr::suffix_check (LookupInfo linf, ParmString word, 
-                                      CheckInfo & ci, GuessInfo * gi,
-                                      int sfxopts, AffEntry * ppfx) const
+bool AffixMgr::suffix_check (const LookupInfo & linf, ParmString word, 
+                             CheckInfo & ci, GuessInfo * gi,
+                             int sfxopts, AffEntry * ppfx) const
 {
-  BasicWordInfo wordinfo;
 
   // first handle the special case of 0 length suffixes
   SfxEntry * se = (SfxEntry *) sStart[0];
   while (se) {
-    wordinfo = se->check(linf, word, ci, gi, sfxopts, ppfx);
-    if (wordinfo) return wordinfo;
+    if (se->check(linf, word, ci, gi, sfxopts, ppfx)) return true;
     se = se->next;
   }
   
   // now handle the general case
-  char * tmpword = myrevstrdup(word);
-  unsigned char sp = *((const unsigned char *)tmpword);
+  unsigned char sp = *((const unsigned char *)(word + word.size() - 1));
   SfxEntry * sptr = (SfxEntry *) sStart[sp];
 
   while (sptr) {
-    if (isSubset(sptr->key(),tmpword)) {
-      wordinfo = sptr->check(linf, word, ci, gi, sfxopts, ppfx);
-      if (wordinfo) {
-        free(tmpword);
-        return wordinfo;
-      }
+    if (isRevSubset(sptr->key(), word + word.size() - 1, word.size())) {
+      if (sptr->check(linf, word, ci, gi, sfxopts, ppfx))
+        return true;
       sptr = sptr->next_eq;
     } else {
       sptr = sptr->next_ne;
     }
   }
     
-  free(tmpword);
-  return NULL;
+  return false;
 }
 
 // check if word with affixes is correctly spelled
-BasicWordInfo AffixMgr::affix_check(LookupInfo linf, ParmString word, 
-                                    CheckInfo & ci, GuessInfo * gi) const
+bool AffixMgr::affix_check(const LookupInfo & linf, ParmString word, 
+                           CheckInfo & ci, GuessInfo * gi) const
 {
-  BasicWordInfo wordinfo;
-
   // Deal With Case in a semi-intelligent manner
   CasePattern cp = case_pattern(*lang,word);
   ParmString pword = word;
@@ -605,17 +596,38 @@ BasicWordInfo AffixMgr::affix_check(LookupInfo linf, ParmString word,
   }
 
   // check all prefixes (also crossed with suffixes if allowed) 
-  wordinfo = prefix_check(linf, pword, ci, gi);
-  if (wordinfo) return wordinfo;
+  if (prefix_check(linf, pword, ci, gi)) return true;
 
   // if still not found check all suffixes
-  wordinfo = suffix_check(linf, sword, ci, gi, 0, NULL);
-  return wordinfo;
+  return suffix_check(linf, sword, ci, gi, 0, NULL);
 }
 
+void AffixMgr::get_word(String & word, const CheckInfo & ci) const
+{
+  CasePattern cp = case_pattern(*lang,word);
+  if (ci.pre_add) {
+    if (cp == FirstUpper) word[0] = lang->to_lower(word[0]);
+    size_t s = strlen(ci.pre_add);
+    word.replace(0, strlen(ci.pre_strip), ci.pre_add, s);
+    if (cp == FirstUpper) word[0] = lang->to_title(word[0]);
+    else if (cp == AllUpper)
+      for (size_t i = 0; i != s; ++i) word[i] = lang->to_upper(word[i]);
+  }
+  if (ci.suf_add) {
+    size_t strip = strlen(ci.suf_strip);
+    size_t s     = strlen(ci.suf_add);
+    size_t start = word.size() - strip;
+    word.replace(start, strip, ci.suf_add, s);
+    if (cp == AllUpper)
+      for (size_t i = start; i != start + s; ++i) 
+	word[i] = lang->to_upper(word[i]);
+  }
+}
+
+
 void AffixMgr::munch(ParmString word, CheckList * cl) const
 {
-  LookupInfo li(0);
+  LookupInfo li(0, LookupInfo::AlwaysTrue);
   CheckInfo ci;
   cl->reset();
   CasePattern cp = case_pattern(*lang,word);
@@ -737,7 +749,7 @@ int AffixMgr::expand(ParmString word, ParmString af,
 // acts like strsep() but only uses a delim char and not 
 // a delim string
 
-char * mystrsep(const char * * stringp, const char delim)
+static char * mystrsep(const char * * stringp, const char delim)
 {
   char * rv = NULL;
   const char * mp = *stringp;
@@ -791,18 +803,28 @@ char * myrevstrdup(const char * s)
   return d; 
 }
 
-
 /* return 1 if s1 is subset of s2 */
-bool isSubset(const char * s1, const char * s2)
+static bool isSubset(const char * s1, const char * s2)
 {
-  int l1 = strlen(s1);
-  int l2 = strlen(s2);
-  if (l1 > l2) return 0;
-  if (strncmp(s2,s1,l1) == 0) return true;
-  return false;
+  while( *s1 && (*s1 == *s2) ) {
+    s1++;
+    s2++;
+  }
+  return (*s1 == '\0');
 }
 
-PosibErr<void> AffixMgr::parse_affix(String & data, 
+// return 1 if s1 (reversed) is a leading subset of end of s2
+static bool isRevSubset(const char * s1, const char * end_of_s2, int len)
+{
+  while( (len > 0) && *s1 && (*s1 == *end_of_s2) ) {
+    s1++;
+    end_of_s2--;
+    len --;
+  }
+  return (*s1 == '\0');
+}
+
+PosibErr<void> AffixMgr::parse_affix(ParmString data, 
                                      const char at, FStream & af)
 {
   int numents = 0;      // number of affentry structures to parse
@@ -810,8 +832,8 @@ PosibErr<void> AffixMgr::parse_affix(String & data,
   short xpflg=0;
   StackPtr<AffEntry> nptr;
 
-  const char * tp = data.c_str();
-  const char * nl = data.c_str();
+  const char * tp = data.str();
+  const char * nl = data.str();
   char * piece;
   int i = 0;
 
@@ -847,11 +869,12 @@ PosibErr<void> AffixMgr::parse_affix(String & data,
     return make_err(bad_file_format, affix_file, msg);
   }
  
-  String key;
+  FixedBuffer<> buf;
+  DataPair datapair;
   // now parse numents affentries for this affix
   for (int j=0; j < numents; j++) {
-    getdata_pair(af, key, data);
-    tp = data.c_str();
+    getdata_pair(af, datapair, buf);
+    tp = datapair.value;
     i = 0;
     np = 0;
 
