@@ -41,17 +41,13 @@ namespace aspeller {
   using acommon::CheckInfo;
   struct GuessInfo;
 
-  struct LookupInfo {
-    SpellerImpl * sp;
-    inline BasicWordInfo lookup (ParmString word);
-    LookupInfo(SpellerImpl * s) : sp(s) {}
-  };
-
   struct CheckList;
   CheckList * new_check_list();
   void delete_check_list(CheckList *);
   CheckInfo * check_list_data(CheckList *);
 
+  struct LookupInfo;
+
   struct AffEntry
   {
     char *       appnd;
@@ -95,15 +91,15 @@ namespace aspeller {
 
     PosibErr<void> setup(ParmString affpath);
 
-    BasicWordInfo       affix_check(LookupInfo, ParmString, CheckInfo &, GuessInfo *) const;
-    BasicWordInfo       prefix_check(LookupInfo, ParmString, CheckInfo &, GuessInfo *) const;
-    BasicWordInfo       suffix_check(LookupInfo, ParmString, CheckInfo &, GuessInfo *,
-				     int sfxopts, AffEntry* ppfx) const;
+    bool affix_check(const LookupInfo &, ParmString, CheckInfo &, GuessInfo *) const;
+    bool prefix_check(const LookupInfo &, ParmString, CheckInfo &, GuessInfo *) const;
+    bool suffix_check(const LookupInfo &, ParmString, CheckInfo &, GuessInfo *,
+                      int sfxopts, AffEntry* ppfx) const;
 
+    void get_word(String & word, const CheckInfo &) const;
 
-    void                munch(ParmString word, CheckList *) const;
-    void                expand(ParmString word, ParmString affixes,
-                               CheckList *) const;
+    void  munch(ParmString word, CheckList *) const;
+    void  expand(ParmString word, ParmString affixes, CheckList *) const;
     // expand enough so the affixes does not effect the first limit
     // characters
     int                 expand(ParmString word, ParmString af, 
@@ -113,7 +109,7 @@ namespace aspeller {
              
   private:
     PosibErr<void> parse_file(const char * affpath);
-    PosibErr<void> parse_affix(String & line, const char at, FStream & af);
+    PosibErr<void> parse_affix(ParmString line, const char at, FStream & af);
 
     void encodeit(AffEntry * ptr, char * cs);
     PosibErr<void> build_pfxlist(PfxEntry* pfxptr);
