@@ -4,12 +4,13 @@
 #define __aspeller_language__
 
 #include "affix.hpp"
-#include "string.hpp"
+#include "cache.hpp"
+#include "config.hpp"
+#include "phonetic.hpp"
 #include "posib_err.hpp"
 #include "stack_ptr.hpp"
-#include "cache.hpp"
-
-#include "phonetic.hpp"
+#include "string.hpp"
+#include "string_buffer.hpp"
 
 using namespace acommon;
 
@@ -20,6 +21,25 @@ namespace acommon {
 
 namespace aspeller {
 
+  struct SuggestRepl {
+    const char * substr;
+    const char * repl;
+  };
+  
+  class SuggestReplEnumeration
+  {
+    const SuggestRepl * i_;
+    const SuggestRepl * end_;
+  public:
+    SuggestReplEnumeration(const SuggestRepl * b, const SuggestRepl * e)
+      : i_(b), end_(e) {}
+    bool at_end() const {return i_ == end_;}
+    const SuggestRepl * next() {
+      if (i_ == end_) return 0;
+      return i_++;
+    }
+  };
+
   class Language : public Cacheable {
   public:
     typedef Config  CacheConfig;
@@ -40,7 +60,6 @@ namespace aspeller {
     String   dir_;
     String   name_;
     String   charset_;
-    String   mid_chars_;
 
     unsigned char to_uchar(char c) const {return static_cast<unsigned char>(c);}
 
@@ -58,11 +77,14 @@ namespace aspeller {
     int max_normalized_;
 
     String      soundslike_chars_;
+    String      stripped_chars_;
 
     StackPtr<Soundslike> soundslike_;
     StackPtr<AffixMgr>   affix_;
+    StackPtr<Config>     lang_config_;
 
-    bool affix_compress_;
+    StringBuffer buf_;
+    Vector<SuggestRepl> repls_;
 
     Language(const Language &);
     void operator=(const Language &);
@@ -70,11 +92,11 @@ namespace aspeller {
   public:
     Language() {}
     PosibErr<void> setup(const String & lang, Config * config);
+    void set_lang_defaults(Config & config);
 
     const char * data_dir() const {return dir_.c_str();}
     const char * name() const {return name_.c_str();}
     const char * charset() const {return charset_.c_str();}
-    const char * mid_chars() const {return mid_chars_.c_str();}
 
     char to_upper(char c) const {return to_upper_[to_uchar(c)];}
     bool is_upper(char c) const {return to_upper(c) == c;}
@@ -114,21 +136,15 @@ namespace aspeller {
       return soundslike_->version();
     }
     
-    bool have_phoneme() const {return false;}
-    String to_phoneme(const char * word) const 
-    {
-      return "";
-    }
-    String to_phoneme(ParmString word) const 
-    {
-      return "";
-    }
-
     const char * soundslike_chars() const {return soundslike_chars_.c_str();}
+    const char * stripped_chars() const {return stripped_chars_.c_str();}
+
+    bool have_soundslike() const {return soundslike_;}
 
     const AffixMgr * affix() const {return affix_;}
 
-    bool affix_compress() const {return affix_compress_;}
+    SuggestReplEnumeration * repl() const {
+      return new SuggestReplEnumeration(repls_.pbegin(), repls_.pend());}
 
     static inline PosibErr<Language *> get_new(const String & lang, Config * config) {
       StackPtr<Language> l(new Language());
@@ -224,7 +240,7 @@ namespace aspeller {
     bool strip_accents;
     ConvertWord(const Language * l = 0)
       : lang(l), strip_accents(false) {}
-    void convert(const String & in, String & out) const
+    void convert(ParmString in, String & out) const
     {
       if (!strip_accents) {
 	out += in;
@@ -322,11 +338,11 @@ namespace aspeller {
       return word;
     }
   }
+
+  String get_stripped_chars(const Language & l);
   
   PosibErr<void> check_if_valid(const Language & l, ParmString word);
 
-  void normalize_mid_characters(const Language & l, String & s);
-
   PosibErr<Language *> new_language(Config &, ParmString lang = 0);
   
 }
