@@ -16,11 +16,22 @@
 // WITH Soundslike data:
 //   (<pad><16 bit: sounds like size><16 bit: offset to next item><soundslike>
 //      (<word><null>)+<null>)+
-// WITHOUT Soundslike data:
+// WITHOUT Soundslike data
 //   (<8 bit: offset><8 bit: sl offset><word><null>[<affix info>]<null>)+
 // compound info is currently not used (ie broken)
 
+// TODO:
+//   consider adding flags to speed up checking
+//     1 bit: all lower case
+//     1 bit: no accent info
+//   which leaves 6 bits for other info which could possible be used
+//   to flag special words.   Such as those which are abbreviations
+//   or prefixes and can only appear before a word if there is a
+//   '-'.
+
 #include <map>
+// FIXME: There is NO real need to use a map here.  Simpily use a vector or list
+//   and then sort when done
 
 using std::pair;
 
@@ -159,33 +170,27 @@ namespace aspeller_default_readonly_ws {
     const SoundslikeJump * jump2;
     WordLookup       word_lookup;
     const char *     word_block;
-    u32int           max_word_length;
-    bool             use_soundslike;
-    bool             affix_info;
     
     ReadOnlyWS(const ReadOnlyWS&);
     ReadOnlyWS& operator= (const ReadOnlyWS&);
 
     struct ElementsParms;
-    struct SoundslikeElementsBase;
-    struct SoundslikeElementsWSL;
-    struct SoundslikeElementsNoSL;
-    struct SoundslikeElementsAffix;
-    struct SoundslikeWords;
-    struct SingleSoundslikeWord;
+    struct SoundslikeElements;
+    struct StrippedElements;
 
   public:
-    VirEmul * detailed_elements() const;
+    WordEntryEnumeration * detailed_elements() const;
     Size      size()     const;
     bool      empty()    const;
 
-    BasicWordInfo convert(const char * w) const {
-      if (affix_info)
-        return BasicWordInfo(w,w+*(w-1),0);
-      else
-        return BasicWordInfo(w,"", 0);
+    static const char * get_affix(const char * w) {return w+(unsigned char)*(w-1);}
+    
+    void convert(const char * w, WordEntry & o) const {
+      o.what = WordEntry::Word;
+      o.word = w;
+      o.aff  = !have_soundslike ? get_affix(w) : w - 1; // w - 1 is NULL
     }
-      
+    
     ReadOnlyWS() {
       block = 0;
     }
@@ -195,16 +200,20 @@ namespace aspeller_default_readonly_ws {
 	if (block_mmaped)
 	  mmap_free(block, block_size);
 	else
-	  delete[] block;
+	  free(block);
       }
     }
       
     PosibErr<void> load(ParmString, Config *, SpellerImpl *, const LocalWordSetInfo *);
-    BasicWordInfo lookup (ParmString word, const SensitiveCompare &) const;
-    VirEmul * words_w_soundslike(const char * soundslike) const;
-    VirEmul * words_w_soundslike(SoundslikeWord soundslike) const;
 
-    VirSoundslikeEmul * soundslike_elements() const;
+    bool lookup(ParmString word, WordEntry &, const SensitiveCompare &) const;
+
+    bool striped_lookup(const char * sondslike, WordEntry &) const;
+
+    bool soundslike_lookup(const WordEntry &, WordEntry &) const;
+    bool soundslike_lookup(const char * sondslike, WordEntry &) const;
+    
+    SoundslikeEnumeration * soundslike_elements() const;
   };
     
   //
@@ -212,20 +221,21 @@ namespace aspeller_default_readonly_ws {
   //
 
   struct ReadOnlyWS::ElementsParms {
-    typedef BasicWordInfo                   Value;
+    typedef WordEntry *                Value;
     typedef WordLookup::const_iterator Iterator; 
     const ReadOnlyWS * ws;
+    WordEntry data;
     ElementsParms(const ReadOnlyWS * w) : ws(w) {}
     bool endf(const Iterator & i) const {return i.at_end();}
     Value end_state() const {return 0;}
-    Value deref(const Iterator & i) const {
-      return ws->convert(ws->word_block + *i);
+    WordEntry * deref(const Iterator & i) {
+      ws->convert(ws->word_block + *i, data);
+      return & data;
     }
   };
 
-  ReadOnlyWS::VirEmul * ReadOnlyWS::detailed_elements() const {
-    return new MakeVirEnumeration<ElementsParms>
-      (word_lookup.begin(), ElementsParms(this));
+  WordEntryEnumeration * ReadOnlyWS::detailed_elements() const {
+    return new MakeEnumeration<ElementsParms>(word_lookup.begin(), ElementsParms(this));
   }
 
   ReadOnlyWS::Size ReadOnlyWS::size() const {
@@ -236,10 +246,14 @@ namespace aspeller_default_readonly_ws {
     return word_lookup.empty();
   }
 
+  static const char * const cur_check_word = "aspell default speller rowl 1.7";
+
   struct DataHead {
     // all sizes except the last four must to divisible by:
     static const uint align = 16;
     char check_word[64];
+    u32int endian_check; // = 12345678
+
     u32int head_size;
     u32int block_size;
     u32int jump1_offset;
@@ -250,14 +264,11 @@ namespace aspeller_default_readonly_ws {
     u32int word_count;
     u32int word_buckets;
     u32int soundslike_count;
-    
-    u32int max_word_length;
 
+    u32int dict_name_size;
     u32int lang_name_size;
     u32int soundslike_name_size;
     u32int soundslike_version_size;
-    u32int minimal_specified;
-    u32int middle_chars_size;
 
     u32int affix_info; // none zero if affix information is encoded in table
   };
@@ -282,75 +293,60 @@ namespace aspeller_default_readonly_ws {
     COUT << "Total Block Size: " << data_head.total_block_size << "\n";
 #endif
 
-    if (strcmp(data_head.check_word, "aspell default speller rowl 1.6") != 0)
+    if (strcmp(data_head.check_word, cur_check_word) != 0)
       return make_err(bad_file_format, fn);
 
-    char * word = new char[data_head.lang_name_size];
-    f.read(word, data_head.lang_name_size);
+    if (data_head.endian_check != 12345678)
+      return make_err(bad_file_format, fn, _("Wrong endian order."));
 
-    PosibErr<void> pe = set_check_lang(word,config);
+    CharVector word;
+
+    word.resize(data_head.dict_name_size);
+    f.read(word.data(), data_head.dict_name_size);
+
+    word.resize(data_head.lang_name_size);
+    f.read(word.data(), data_head.lang_name_size);
+
+    PosibErr<void> pe = set_check_lang(word.data(),config);
     if (pe.has_err())
       return pe.with_file(fn);
 
-    delete[] word;
+    if (data_head.soundslike_name_size != 0) {
+      have_soundslike = true;
 
-    word = new char[data_head.soundslike_name_size];
-    f.read(word, data_head.soundslike_name_size);
+      word.resize(data_head.soundslike_name_size);
+      f.read(word.data(), data_head.soundslike_name_size);
 
-    if (strcmp(word, lang()->soundslike_name()) != 0)
-      return make_err(bad_file_format, fn, "Wrong Soundslike");
-    if (strcmp(word, "none") == 0)
-      use_soundslike=false;
-    else
-      use_soundslike=true;
+      if (strcmp(word.data(), lang()->soundslike_name()) != 0)
+        return make_err(bad_file_format, fn, _("Wrong soundslike"));
 
-    delete[] word;
+      word.resize(data_head.soundslike_version_size);
+      f.read(word.data(), data_head.soundslike_version_size);
 
-    word = new char[data_head.soundslike_version_size];
-    f.read(word, data_head.soundslike_version_size);
-
-    if (strcmp(word, lang()->soundslike_version()) != 0)
-      return make_err(bad_file_format, fn, "Wrong Soundslike Version");
-
-    delete[] word;
-
-    affix_info = data_head.affix_info;
-
-    if (data_head.minimal_specified != u32int_max) {
-      word = new char[data_head.middle_chars_size];
-      f.read(word, data_head.middle_chars_size);
-      
-      if (strcmp(word, lang()->mid_chars()) != 0)
-	return make_err(bad_file_format, fn, "Different Middle Characters");
-      
-      delete[] word;
-
-      if (data_head.minimal_specified != u32int_max) {
-	config->replace("run-together-specified", "true");
-	unsigned int m = config->retrieve_int("minimal-specified-component");
-	if (data_head.minimal_specified < m) {
-	  char buf[20];
-	  sprintf(buf, "%i", data_head.minimal_specified);
-	  config->replace("minimal-specified-component", buf);
-	}
-      }
+      if (strcmp(word.data(), lang()->soundslike_version()) != 0)
+        return make_err(bad_file_format, fn, _("Wrong soundslike version"));
     }
 
+    affix_compressed = data_head.affix_info;
 
     block_size = data_head.block_size;
     block = mmap_open(block_size, f, data_head.head_size);
     block_mmaped = block != (char *)MAP_FAILED;
     if (!block_mmaped) {
-      block = new char[block_size];
+      block = (char *)malloc(block_size);
       f.seek(data_head.head_size);
       f.read(block, block_size);
     }
 
-    jump1 = reinterpret_cast<const SoundslikeJump *>(block 
-						     + data_head.jump1_offset);
-
-    jump2 = reinterpret_cast<const SoundslikeJump *>(block 
-						     + data_head.jump2_offset);
+    if (data_head.jump2_offset) {
+      fast_scan = true;
+      jump1 = reinterpret_cast<const SoundslikeJump *>(block 
+						       + data_head.jump1_offset);
+      jump2 = reinterpret_cast<const SoundslikeJump *>(block 
+						       + data_head.jump2_offset);
+    } else {
+      jump1 = jump2 = 0;
+    }
 
     word_block       = block + data_head.word_offset;
 
@@ -362,158 +358,60 @@ namespace aspeller_default_readonly_ws {
     word_lookup.vector().set(begin, begin + data_head.word_buckets);
     word_lookup.set_size(data_head.word_count);
     
-    max_word_length = data_head.max_word_length;
-    
     return no_err;
   }
 
-  BasicWordInfo ReadOnlyWS::lookup(ParmString word, 
-				   const SensitiveCompare & c) const 
+  bool ReadOnlyWS::lookup(ParmString word, WordEntry & o,
+                          const SensitiveCompare & c) const 
   {
+    o.clear();
+    o.what = WordEntry::Word;
     WordLookup::ConstFindIterator i = word_lookup.multi_find(word);
     for (; !i.at_end(); i.adv()) {
       const char * w = word_block + i.deref();
-      if (c(word, w)) 
-        return convert(w);
+      if (c(word, w)) {
+        convert(w,o);
+        return true;
+      }
     }
-    return 0;
+    return false;
   }
 
-  struct ReadOnlyWS::SoundslikeWords : public VirEnumeration<BasicWordInfo> {
-    const char * cur;
-    SoundslikeWords(const char * c) : cur(c) {}
-    Value next() {
-      if (*cur == 0)
-	return 0;
-      Value tmp(cur, *(cur - 1)); 
-      unsigned int len = strlen(cur);
-      cur += len + 1;
-      return tmp;
-    }
-    bool at_end() const {return *cur == 0;}
-    Base * clone() const {return new SoundslikeWords(*this);}
-    void assign(const Base * other) 
-      {*this = *static_cast<const SoundslikeWords *>(other);}
-  };
-
-  struct ReadOnlyWS::SingleSoundslikeWord : public VirEnumeration<BasicWordInfo> {
-    String cur;
-    bool done;
-    SingleSoundslikeWord(const char * c) : cur(c), done(false) {}
-    Value next() {
-      if (done) return 0;
-      done = true;
-      //CERR << "$$" << cur << '\n';
-      return cur.c_str();
-    }
-    bool at_end() const {return done;}
-    Base * clone() const {return new SingleSoundslikeWord(*this);}
-    void assign(const Base * other) 
-      {*this = *static_cast<const SingleSoundslikeWord *>(other);}
-  };
-
-  struct ReadOnlyWS::SoundslikeElementsBase : public VirSoundslikeEmul
+  struct ReadOnlyWS::SoundslikeElements : public SoundslikeEnumeration
   {
-    int level;
+    WordEntry data;
     const ReadOnlyWS * obj;
     const SoundslikeJump * jump1;
     const SoundslikeJump * jump2;
     const char * cur;
-    const char * tmp;
+    int level;
+    bool sl;
 
-    virtual u16int next_pos() const = 0;
-
-    const char * adv(int stopped_at);
-
-    SoundslikeElementsBase(const ReadOnlyWS * o)
-      : level(1), obj(o), jump1(obj->jump1), jump2(obj->jump2), cur(0) {}
-  };
-
-  struct ReadOnlyWS::SoundslikeElementsWSL : public SoundslikeElementsBase
-  {
-    SoundslikeElementsWSL(const ReadOnlyWS * o)
-      : SoundslikeElementsBase(o) {}
-
-    u16int next_pos() const
-      {return *reinterpret_cast<const u16int *>(cur - 2);}
-
-    SoundslikeWord next(int stopped_at) {
-      const char * r = adv(stopped_at);
-      if (r) return SoundslikeWord(r,0);
-      return SoundslikeWord(tmp,tmp);
+    u16int next_pos() const {
+      if (sl) return *reinterpret_cast<const u16int *>(cur - 2);
+      else return *reinterpret_cast<const unsigned char *>(cur - 2);
     }
+
+    WordEntry * next(int stopped_at);
+
+    SoundslikeElements(const ReadOnlyWS * o)
+      : obj(o), jump1(obj->jump1), jump2(obj->jump2), cur(0), 
+        level(1), sl(o->have_soundslike) {
+      data.what = o->have_soundslike ? WordEntry::Soundslike : WordEntry::Word;}
   };
+  
+  WordEntry * ReadOnlyWS::SoundslikeElements::next(int stopped_at) {
 
-  struct ReadOnlyWS::SoundslikeElementsNoSL : public SoundslikeElementsBase
-  {
-    CharVector buf;
-    const Language * lang;
+    //CERR << level << ":" << stopped_at << "  :";
+    //CERR << jump1->sl << ":" << jump2->sl << "\n";
 
-    SoundslikeElementsNoSL(const ReadOnlyWS * o, const Language * l)
-      : SoundslikeElementsBase(o), lang(l) {}
-
-    u16int next_pos() const
-      {return *reinterpret_cast<const unsigned char *>(cur - 2);}
-
-    SoundslikeWord next(int stopped_at) {
-      const char * r = adv(stopped_at);
-      if (r) return SoundslikeWord(r,0);
-      buf.clear();
-      to_stripped(*lang, tmp, buf);
-      buf.append('\0');
-      return SoundslikeWord(buf.data(), tmp);
-    }
-  };
-
-  struct ReadOnlyWS::SoundslikeElementsAffix : public SoundslikeElementsBase
-  {
-    CharVector buf;
-    const Language * lang;
-    CheckList cl;
-    const CheckInfo * ci_cur;
-
-    SoundslikeElementsAffix(const ReadOnlyWS * o, const Language * l)
-      : SoundslikeElementsBase(o), lang(l), ci_cur(0) {}
-
-    u16int next_pos() const
-      {return *reinterpret_cast<const unsigned char *>(cur - 2);}
-
-    SoundslikeWord next(int stopped_at) {
-      if (!ci_cur) {
-        const char * r = adv(stopped_at);
-        //if (r) CERR << ",," << r << '\n';
-        if (r) return SoundslikeWord(r,0);
-        BasicWordInfo w = obj->convert(tmp);
-        //CERR << ".." << w.word << '/' << w.affixes << '\n';
-        cl.reset();
-        lang->affix()->expand(w.word, w.affixes, &cl);
-        ci_cur = cl.data + 1;
-      }
-
-      const CheckInfo * ci_tmp = ci_cur;
-      ci_cur = ci_cur->next;
-
-      buf.clear();
-      to_stripped(*lang, ci_tmp->word, buf);
-      buf.append('\0');
-      buf.append(ci_tmp->word, strlen(ci_tmp->word));
-      buf.append('\0');
-      //CERR << "::" << ci_tmp->word << ' ' << buf.data() << '\n';
-      return SoundslikeWord(buf.data(), tmp);
-    }
-  };
-
-  const char * ReadOnlyWS::SoundslikeElementsBase::adv(int stopped_at) {
-
-    //COUT << level << ":" << stopped_at << "  ";
-    //COUT << jump1->sl << " " << jump2->sl << "\n";
-
-    tmp = cur;
+    const char * tmp = cur;
 
     if (level == 1 && stopped_at < 2) {
 
       ++jump1;
-      return jump1->sl;
+      tmp = jump1->sl;
+      goto jquit;
 	  
     } else if (level == 2 && stopped_at < 3) {
 
@@ -521,16 +419,18 @@ namespace aspeller_default_readonly_ws {
       if (jump2[-1].sl[1] != jump2[0].sl[1]) {
         ++jump1;
         level = 1;
-        return jump1->sl;
+        tmp = jump1->sl;
       } else {
-        return jump2->sl;
+        tmp = jump2->sl;
       }
+      goto jquit;
       
     } else if (level == 1) {
 
       level = 2;
       jump2 = obj->jump2 + jump1->loc;
-      return jump2->sl;
+      tmp = jump2->sl;
+      goto jquit;
 
     } else if (level == 2) {
 
@@ -540,17 +440,16 @@ namespace aspeller_default_readonly_ws {
 
     } else if (next_pos() == 0) {
 
-      //cur += 2;
-      cur = 0; // REMOVE ME When sure that cur it not used...
       level = 2;
       ++jump2;
       if (jump2[-1].sl[1] != jump2[0].sl[1]) {
         level = 1;
         ++jump1;
-        return jump1->sl;
+        tmp = jump1->sl;
       } else {
-        return jump2->sl;
+        tmp = jump2->sl;
       }
+      goto jquit;
 
     } else {
 
@@ -558,55 +457,117 @@ namespace aspeller_default_readonly_ws {
       
     }
 
-    return 0;
+    data.word = tmp;
+    if (!sl) {
+      data.what = WordEntry::Word;
+      data.aff  = obj->get_affix(tmp);
+    }
+    data.intr[0] = (void *)tmp;
+    return &data;
+
+  jquit:
+    if (!*tmp) return 0;
+    data.word = tmp;
+    data.intr[0] = 0;
+    if (!sl) {
+      data.what = WordEntry::Stripped;
+      data.aff  = 0;
+    }
+    return &data;
   }
 
+  struct ReadOnlyWS::StrippedElements : public SoundslikeEnumeration
+  {
+    WordEntry data;
+    const char * cur;
+
+    u16int next_pos() const {
+      return *reinterpret_cast<const unsigned char *>(cur - 2);
+    }
+
+    WordEntry * next(int stopped_at);
+
+    StrippedElements(const ReadOnlyWS * o)
+      : cur(o->word_block + 2) {data.what = WordEntry::Word;}
+  };
+
+  WordEntry * ReadOnlyWS::StrippedElements::next(int) {
+
+    const char * tmp = cur;
+    cur += next_pos();
+    if (cur == tmp) return 0;
+    data.intr[0] = (void *)tmp;
+    data.word = tmp;
+    data.aff  = ReadOnlyWS::get_affix(tmp);
+    return &data;
+
+  }
+
+  SoundslikeEnumeration * ReadOnlyWS::soundslike_elements() const {
+
+    if (jump1)
+      return new SoundslikeElements(this);
+    else
+      return new StrippedElements(this);
+
+  }
     
-  ReadOnlyWS::VirSoundslikeEmul * ReadOnlyWS::soundslike_elements() const {
+  static void soundslike_next(WordEntry * w)
+  {
+    const char * cur = static_cast<const char *>(w->intr[0]);
+    w->word = cur;
+    unsigned int len = strlen(cur);
+    cur += len + 1;
+    w->intr[0] = (void *)cur;
+    if (*cur == 0) w->adv_ = 0;
+  }
 
-    if (use_soundslike) {
-      
-      return new SoundslikeElementsWSL(this);
-
-    } else if (affix_info) {
-
-      return new SoundslikeElementsAffix(this, lang());
+  //static 
+  //void ReadOnlyWS::stripped_next(WordEntry * w) {}
 
+  bool ReadOnlyWS::striped_lookup(const char * sl, WordEntry & w) const
+  {
+    w.clear();
+    WordLookup::ConstFindIterator i = word_lookup.multi_find(sl);
+    if (i.at_end()) return false;
+    convert(word_block + i.deref(), w);
+    return true;
+    // FIXME deal with multiple words 
+    //   the lookup should point to the first one of the kind ....
+  }
+    
+  bool ReadOnlyWS::soundslike_lookup(const char * sl, WordEntry & w) const 
+  {
+    if (!have_soundslike) {
+      return ReadOnlyWS::striped_lookup(sl,w);
     } else {
-
-      return new SoundslikeElementsNoSL(this, lang());
-      
+      return false;
     }
   }
-    
-  ReadOnlyWS::VirEmul * 
-  ReadOnlyWS::words_w_soundslike(const char * soundslike) const {
 
-    abort();
+  bool ReadOnlyWS::soundslike_lookup(const WordEntry & s, WordEntry & w) const 
+  {
+    w.clear();
 
-  }
-  
-  ReadOnlyWS::VirEmul *
-  ReadOnlyWS::words_w_soundslike(SoundslikeWord w) const {
+    if (s.intr[0] == 0) {
 
-    if (w.word_list_pointer == 0) {
+      return false;
 
-      return new SoundslikeWords("");
-
-    } else if (use_soundslike) {
+    } else if (have_soundslike) {
       
-      u16int sl_size = *reinterpret_cast<const u16int *>(w.soundslike-4);
-      
-      return new SoundslikeWords(w.soundslike + sl_size + 1);
-
-    } else if (affix_info) {
-
-      return new SingleSoundslikeWord(w.soundslike + strlen(w.soundslike) + 1);
+      w.what = WordEntry::Word;
+      u16int sl_size = *reinterpret_cast<const u16int *>(s.word-4);
+      w.intr[0] = (void *)(s.word + sl_size + 1);
+      w.adv_ = soundslike_next;
+      soundslike_next(&w);
+      return true;
       
     } else {
-      
-      return new SingleSoundslikeWord(static_cast<const char *>
-                                      (w.word_list_pointer));
+
+      w.what = WordEntry::Word;
+      convert(s.word, w);
+      return true;
+
     }
 
   }
@@ -627,8 +588,7 @@ namespace aspeller_default_readonly_ws {
 
   struct WordData {
     static const int size = 3;
-    char compound_info;
-    char offset;
+    char total_size; // including null chars
     char affix_offset;
     char data[];
     const char * affix_data() const {return data + affix_offset;}
@@ -691,47 +651,58 @@ namespace aspeller_default_readonly_ws {
       out << '\0';
   }
 
-  PosibErr<void> create (ParmString base, 
-			 StringEnumeration * els,
-			 const Language & lang) 
+  PosibErr<void> create (StringEnumeration * els,
+			 const Language & lang,
+                         Config & config) 
   {
     assert(sizeof(u16int) == 2);
     assert(sizeof(u32int) == 4);
 
-    bool affix_compress = lang.affix_compress();
+    bool affix_compress = (lang.affix() && 
+                           config.retrieve_bool("affix-compress"));
     
-    bool use_soundslike=true;
-    if (strcmp(lang.soundslike_name(),"none") == 0)
-      use_soundslike=false;
+    bool use_soundslike = (lang.have_soundslike() &&
+                           config.retrieve_bool("use-soundslike"));
+   
+    bool use_jump_tables = use_soundslike || config.retrieve_bool("use-jump-tables");
 
-    assert(!(affix_compress && use_soundslike));
-    // FIXME: return error
+    CERR << (affix_compress ? "  AFFIX COMPRESS" : "")
+         << (use_soundslike ? "  USING SOUNDSLIKE" : "") 
+         << (use_jump_tables ? "  USING JUMP TABLES" : "")
+         << "\n";
 
-    const char * mid_chars = lang.mid_chars();
+    assert(!(affix_compress && use_soundslike)); // FIXME: return error
+
+    String base = config.retrieve("master-path");
 
     DataHead data_head;
     memset(&data_head, 0, sizeof(data_head));
-    strcpy(data_head.check_word, "aspell default speller rowl 1.6");
+    strcpy(data_head.check_word, cur_check_word);
 
-    data_head.lang_name_size          = strlen(lang.name()) + 1;
-    data_head.soundslike_name_size    = strlen(lang.soundslike_name()) + 1;
-    data_head.soundslike_version_size = strlen(lang.soundslike_version()) + 1;
-    data_head.middle_chars_size       = strlen(mid_chars) + 1;
+    data_head.endian_check = 12345678;
+
+    data_head.dict_name_size = 1;
+    data_head.lang_name_size = strlen(lang.name()) + 1;
+    if (use_soundslike) {
+      data_head.soundslike_name_size    = strlen(lang.soundslike_name()) + 1;
+      data_head.soundslike_version_size = strlen(lang.soundslike_version()) + 1;
+    }
     data_head.head_size  = sizeof(DataHead);
+    data_head.head_size += data_head.dict_name_size;
     data_head.head_size += data_head.lang_name_size;
     data_head.head_size += data_head.soundslike_name_size;
     data_head.head_size += data_head.soundslike_version_size;
     data_head.head_size  = round_up(data_head.head_size, DataHead::align);
 
-    data_head.minimal_specified = u32int_max;
-
     data_head.affix_info = affix_compress;
 
     String temp;
 
-    // these are the final two data structures
+    // these are the final data structures
     CharVector     data;
     Vector<u32int> final_hash;
+    Vector<SoundslikeJump> jump1;
+    Vector<SoundslikeJump> jump2;
 
     int num_entries = 0;
     
@@ -764,17 +735,7 @@ namespace aspeller_default_readonly_ws {
 	char * w = tstr.data();
 
         char * p0 = strchr(w, '/');
-	char * p1 = strchr(w, ':');
 
-        if (p0 && p1 && !(p0 < p1))
-          abort(); // FIXME return error
-
-        if (p1) { // compound info, handled first to get size (s) right
-          s = p1 - w;
-	  *p1 = '\0';
-	  ++p1;
-	}
-	
         if (p0) { // affix info
 	  s = p0 - w;
           *p0 = '\0';
@@ -788,12 +749,6 @@ namespace aspeller_default_readonly_ws {
         if (affixes && !lang.affix())
           abort(); // FIXME return error
 
-	// Read in compound info
-	
-	CompoundInfo c;
-        if (p1 && *c.read(p1, lang) != '\0')
-	  return make_err(invalid_flag, w, p1);
-	
         if (!affixes || (affixes && !affix_compress)) {
 
           // Now expand any affix compression
@@ -821,13 +776,8 @@ namespace aspeller_default_readonly_ws {
             }
             if (dup) continue;
           
-            if(s > data_head.max_word_length)
-              data_head.max_word_length = s;
-            if (c.any() && s < data_head.minimal_specified)
-              data_head.minimal_specified = s;
             WordData * b = (WordData *)buf.alloc(s + WordData::size + 2);
-            b->compound_info = static_cast<char>(c.d);
-            b->offset = s + 2; // this only used when no sl data
+            b->total_size = s + 2;
             b->affix_offset = s + 1;
             memcpy(b->data, w, s+1);
             b->data[s+1] = '\0'; // needed to keep things simple if sl data not used
@@ -837,18 +787,27 @@ namespace aspeller_default_readonly_ws {
 
         } else {
 
-          // expand any affixes which will effect the first
-          // 3 letters of a word.  This is needed so that the
-          // jump tables will function correctly
+          int n;
+          WordAff * p, * end;
+          WordAff wa_buf;
 
-          af_list.resize(strlen(affixes) * strlen(affixes) + 1);
-          int n = lang.affix()->expand(w,affixes,3,af_list.data());
+          if (use_jump_tables) {
+            // expand any affixes which will effect the first
+            // 3 letters of a word.  This is needed so that the
+            // jump tables will function correctly
+            af_list.resize(strlen(affixes) * strlen(affixes) + 1);
+            n = lang.affix()->expand(w,affixes,3,af_list.data());
+            p = af_list.data();
+            end = p + n;
+          } else {
+            wa_buf.word = w;
+            wa_buf.af   = affixes;
+            p = & wa_buf;
+            end = p + 1;
+          }
 
-          // now interate through the expanded list and insert
+          // now interate through the list and insert
           // the words 
-
-          WordAff * p = af_list.data();
-          WordAff * end = p + n;
           for (; p != end; ++p)
           {
             //CERR << p->word << '/' << p->af << '\n';
@@ -860,22 +819,16 @@ namespace aspeller_default_readonly_ws {
                 if (!p->af.empty())
                   CERR << "WARNING: Ignoring duplicate: " << w << '\n';
                 dup = true;
-                // FiMXE: deal with duplicates properly
+                // FIXME: deal with duplicates properly
               }
             }
             if (dup) continue;
 
-            if(s > data_head.max_word_length)
-              data_head.max_word_length = s;
-            if (c.any() && s < data_head.minimal_specified)
-              data_head.minimal_specified = s;
-            
             WordData * b;
             b = (WordData *)buf.alloc(s + WordData::size + p->af.size() + 2);
-            b->offset = s + 1 + p->af.size() + 1;
+            b->total_size = s + 1 + p->af.size() + 1;
             b->affix_offset = s + 1;
 
-            b->compound_info = static_cast<char>(c.d);
             memcpy(b->data, w, s + 1);
             memcpy(b->data + s + 1, p->af.c_str(), p->af.size() + 1);
 
@@ -949,8 +902,10 @@ namespace aspeller_default_readonly_ws {
 	}
 	++j->second.num_inserted;
       }
+
+      data_head.soundslike_count = sound_map.size();
       
-    } else {
+    } else if (use_jump_tables) {
       
       for (unsigned int i = 0; i != word_hash->vector().size(); ++i) {
 
@@ -966,6 +921,7 @@ namespace aspeller_default_readonly_ws {
         sl.d.single.hash_idx = i;
         sl.num_inserted      = 1;
       }
+
     }
 
     data_head.word_count   = word_hash->size();
@@ -974,16 +930,15 @@ namespace aspeller_default_readonly_ws {
     final_hash.insert(final_hash.begin(), 
 		      word_hash->bucket_count(), u32int_max);
     
-    word_hash.del(); // the word_hash is no longer needed so free the space
-    
     //
     // Create the data block which includes both the word and soundlike
     //  data
     //
 
-    Vector<SoundslikeJump> jump1;
-    Vector<SoundslikeJump> jump2;
-    {
+    if (use_jump_tables) {
+
+      word_hash.del(); // the word_hash is no longer needed so free the space
+
       SoundMap::iterator i   = sound_map.begin();
       SoundMap::iterator end = sound_map.end();
 
@@ -999,8 +954,7 @@ namespace aspeller_default_readonly_ws {
 	if (use_soundslike) {
 
 	  sl = i->first;
-          while (data.size() % 2 != 0)
-            data.write('\0');
+          if (data.size() % 2 != 0) data.write('\0');
 	  data.write16(sl.size());
           data.write16(0); // place holder for offset to next item
 
@@ -1054,14 +1008,9 @@ namespace aspeller_default_readonly_ws {
 	  SoundslikeList::Word * end = k + i->second.size;
           assert(use_soundslike || i->second.size == 1);
 	  for (;k != end; ++k) {
-	    //if (k->d->compound_info)
-	    //  data << k->d->compound_info;
-
-            //CERR << (int)k->d->affix_offset << '\n';
 	    final_hash[k->hash_idx] = data.size();
 
             if (!use_soundslike) {
-              //data[prev_pos - 2] = k->d->offset;
               data[prev_pos - 1] = k->d->affix_offset;
             }
 
@@ -1078,6 +1027,7 @@ namespace aspeller_default_readonly_ws {
       }
 
       // add special end case
+      if (data.size() % 2 != 0) data.write('\0');
       data.write16(0);
       data.write16(0);
       if (use_soundslike)
@@ -1087,22 +1037,48 @@ namespace aspeller_default_readonly_ws {
       data.at32(4) = 0;
       jump2.push_back(SoundslikeJump());
       jump1.push_back(SoundslikeJump());
-    }
 
-    data_head.soundslike_count   = sound_map.size();
+    } else {
+
+      // No jump tables, just write the words in the order that
+      // they appear in the hash table.
+
+      for (unsigned int i = 0; i != word_hash->vector().size(); ++i) {
+
+        const WordData * value = word_hash->vector()[i];
+        
+        if (word_hash->parms().is_nonexistent(value)) continue;
+
+        data.write(value->total_size + 2);
+        data.write(value->affix_offset);
+
+        final_hash[i] = data.size();
+
+        data.write(value->data, value->total_size);
+        
+      }
+
+      data.write(0);
+      data.write(0);
+      data.write(0);
+
+    }
 
     FStream out;
     out.open(base, "wb");
 
-    // Write jump1 table
     advance_file(out, data_head.head_size);
-    data_head.jump1_offset = out.tell() - data_head.head_size;
-    out.write(jump1.data(), jump1.size() * sizeof(SoundslikeJump));
 
-    // Write jump2 table
-    advance_file(out, round_up(out.tell(), DataHead::align));
-    data_head.jump2_offset = out.tell() - data_head.head_size;
-    out.write(jump2.data(), jump2.size() * sizeof(SoundslikeJump));
+    if (use_jump_tables) {
+      // Write jump1 table
+      data_head.jump1_offset = out.tell() - data_head.head_size;
+      out.write(jump1.data(), jump1.size() * sizeof(SoundslikeJump));
+      
+      // Write jump2 table
+      advance_file(out, round_up(out.tell(), DataHead::align));
+      data_head.jump2_offset = out.tell() - data_head.head_size;
+      out.write(jump2.data(), jump2.size() * sizeof(SoundslikeJump));
+    } 
 
     // Write data block
     advance_file(out, round_up(out.tell(), DataHead::align));
@@ -1114,16 +1090,19 @@ namespace aspeller_default_readonly_ws {
     data_head.hash_offset = out.tell() - data_head.head_size;
     out.write(&final_hash.front(), final_hash.size() * 4);
     
+    // calculate block size
     advance_file(out, round_up(out.tell(), DataHead::align));
     data_head.block_size = out.tell() - data_head.head_size;
 
     // write data head to file
     out.seek(0);
     out.write(&data_head, sizeof(DataHead));
+    out.write(" ", 1);
     out.write(lang.name(), data_head.lang_name_size);
-    out.write(lang.soundslike_name(), data_head.soundslike_name_size);
-    out.write(lang.soundslike_version(), data_head.soundslike_version_size);
-    out.write(mid_chars, data_head.middle_chars_size); 
+    if (use_soundslike) {
+      out.write(lang.soundslike_name(), data_head.soundslike_name_size);
+      out.write(lang.soundslike_version(), data_head.soundslike_version_size);
+    }
 
     return no_err;
   }
@@ -1138,8 +1117,9 @@ namespace aspeller {
     PosibErr<Language *> res = new_language(config);
     if (!res) return res;
     lang.reset(res.data);
-    aspeller_default_readonly_ws::create(config.retrieve("master-path"),
-                                         els,*lang);
+    lang->set_lang_defaults(config);
+    aspeller_default_readonly_ws::create(els,*lang,config);
     return no_err;
   }
 }
+
