@@ -33,6 +33,9 @@
 // means.  For more information on the metaphone algorithm please see
 // the file metaphone.cc which included a detailed description of it.
 
+// FIXME: avoid copying around strings
+//        instead write the word directly to its final location
+
 #include "getdata.hpp"
 
 #include "fstream.hpp"
@@ -54,8 +57,7 @@
 #include "stack_ptr.hpp"
 #include "suggest.hpp"
 
-
-//#include "iostream.hpp"
+#include "iostream.hpp"
 //#define DEBUG_SUGGEST
 
 using namespace aspeller;
@@ -77,17 +79,15 @@ namespace aspeller_default_suggest {
   //
   struct OriginalWord {
     String   word;
-    String   word_stripped;
+    String   stripped;
     String   soundslike;
-    String   phoneme;
     CasePattern  case_pattern;
     OriginalWord() {}
-    OriginalWord (const String &w, const String &sl, const String &p)
-      : word(w), soundslike(sl), phoneme(p) {}
-    OriginalWord (const String &w, const String &sl, const String &p,
+    OriginalWord (const String &w, const String &sl)
+      : word(w), soundslike(sl) {}
+    OriginalWord (const String &w, const String &sl,
 		 const String &l, CasePattern cp)
-      : word(w), word_stripped(l), soundslike(sl), phoneme(p), case_pattern(cp)
-    {}
+      : word(w), stripped(l), soundslike(sl), case_pattern(cp) {}
   };
 
   //
@@ -101,7 +101,7 @@ namespace aspeller_default_suggest {
     int           score;
     int           soundslike_score;
     bool          count;
-    ReplacementList::VirEmul * repl_list;
+    WordEntry * repl_list;
     ScoreWordSound() {repl_list = 0;}
     ~ScoreWordSound() {delete repl_list;}
   };
@@ -134,15 +134,14 @@ namespace aspeller_default_suggest {
   class Score {
   protected:
     const Language * lang;
-    OriginalWord      original_word;
+    OriginalWord     original_word;
     SuggestParms     parms;
 
   public:
     Score(const Language *l, const String &w, const SuggestParms & p)
       : lang(l), original_word(w, l->to_soundslike(w.c_str()), 
-			      l->to_phoneme(w.c_str()),
-			      to_stripped(*l, w),
-			      case_pattern(*l, w)),
+			       to_stripped(*l, w),
+			       case_pattern(*l, w)),
       parms(p)
     {}
     String fix_case(const String & word) {
@@ -163,14 +162,24 @@ namespace aspeller_default_suggest {
 
     BasicList<String>      strings;
 
+    bool use_soundslike, fast_scan, fast_lookup, affix_compress_soundslike;
+
     static const bool do_count = true;
     static const bool dont_count = false;
     static const bool do_need_alloc = true;
     static const bool dont_need_alloc = false;
 
-    void try_sound(const char *, int ms);
-    void add_nearmiss(const char * word, int ms, bool count, 
-		      bool need_alloc, ReplacementList::VirEmul * rl = 0) {
+    const String & active_soundslike() {
+      return use_soundslike ? original_word.soundslike : original_word.stripped;
+    };
+    const char * soundslike_chars() {
+      return use_soundslike ? lang->soundslike_chars() : lang->stripped_chars();
+    }
+
+    void try_sound(ParmString, int score);
+    void add_nearmiss(ParmString word, int score, bool count, 
+		      bool need_alloc, WordEntry * rl = 0) 
+    {
       near_misses.push_front(ScoreWordSound());
       ScoreWordSound & d = near_misses.front();
       if (need_alloc) {
@@ -181,10 +190,8 @@ namespace aspeller_default_suggest {
       }
 
       if (parms.use_typo_analysis) {
-
 	unsigned int l = strlen(word);
 	if (l > max_word_length) max_word_length = l;
-	
       }
 
       if (!is_stripped(*lang,word)) {
@@ -194,7 +201,7 @@ namespace aspeller_default_suggest {
 	d.word_stripped = d.word;
       }
 
-      d.soundslike_score = ms;
+      d.soundslike_score = score;
       d.count = count;
       d.repl_list = rl;
     }
@@ -226,14 +233,21 @@ namespace aspeller_default_suggest {
     }
 
     void try_others();
+    void try_split();
+    void try_one_edit();
+    void try_scan();
+    void try_repl();
+
     void score_list();
     void transfer();
   public:
     Working(SpellerImpl * m, const Language *l,
 	    const String & w, const SuggestParms & p)
-      : Score(l,w,p), threshold(1), max_word_length(0), speller(m) {}
+      : Score(l,w,p), threshold(1), max_word_length(0), speller(m) ,
+        use_soundslike(m->use_soundslike),
+        fast_scan(m->fast_scan), fast_lookup(m->fast_lookup),
+	affix_compress_soundslike(!m->suggest_affix_ws.empty()) {}
     void get_suggestions(NearMissesFinal &sug);
-    void get_suggestions_ultra(NearMissesFinal &sug);
   };
 
   //
@@ -241,35 +255,54 @@ namespace aspeller_default_suggest {
   //    the possable words to near_misses
   //
 
-  void Working::try_sound (const char * m, int ms)  
+  void Working::get_suggestions(NearMissesFinal & sug) {
+    near_misses_final = & sug;
+    if (active_soundslike().empty()) return;
+    try_others();
+    score_list();
+    transfer();
+  }
+  
+  void Working::try_sound(ParmString str, int score)  
   {
-    // sound is the object in the list which is a lot smaller than m
-
-    for (SpellerImpl::DataSetCollection::const_iterator i 
-	   = speller->data_set_collection().begin();
-	 i != speller->data_set_collection().end();
-	 ++i) {
+    String word;
+    WordEntry sw;
+    for (SpellerImpl::WS::const_iterator i = speller->suggest_ws.begin();
+         i != speller->suggest_ws.end();
+         ++i)
+    {
+      i->ws->soundslike_lookup(str, sw);
+      for (;!sw.at_end(); sw.adv()) {
+        word.clear();
+        i->convert.convert(sw.word, word);
+        WordEntry * repl = 0;
+        if (sw.what == WordEntry::Misspelled) {
+          repl = new WordEntry;
+          const BasicReplacementSet * repl_set
+            = static_cast<const BasicReplacementSet *>(i->ws);
+          repl_set->repl_lookup(sw, *repl);
+        }
+        add_nearmiss(word, score, do_count, do_need_alloc, repl);
+      }
+    }
+    if (affix_compress_soundslike) {
+      CheckInfo ci; memset(&ci, 0, sizeof(ci));
+      bool res = lang->affix()->affix_check(LookupInfo(speller, LookupInfo::Soundslike), str, ci, 0);
+      if (!res) return;
       
-      if (!i->use_to_suggest) continue;
-
-      if (i->data_set->basic_type == DataSet::basic_word_set) {
-
-	BasicWordSet::Emul e = static_cast<const BasicWordSet *>
-	  (i->data_set)->words_w_soundslike(m);
-	BasicWordInfo w;
-	String word;
-	while ((w = e.next())) {
-	  w.get_word(word, i->local_info.convert);
-	  add_nearmiss(word.c_str(), ms, do_count, do_need_alloc);
+      // FIXME: This is not completely correct when there are multiple
+      //   words for a single stripped word.
+      for (SpellerImpl::WS::const_iterator i = speller->suggest_affix_ws.begin();
+	   i != speller->suggest_affix_ws.end();
+	   ++i) 
+      {
+	i->ws->soundslike_lookup(ci.word, sw);
+	for (;!sw.at_end(); sw.adv()) {
+	  word.clear();
+	  i->convert.convert(sw.word, word);
+	  lang->affix()->get_word(word, ci);
+	  add_nearmiss(word.c_str(), score, do_count, do_need_alloc);
 	}
-	
-      } else {
-
-	BasicReplacementSet::Emul e = static_cast<const BasicReplacementSet *>(i->data_set)->repls_w_soundslike(m);
-	ReplacementList repl;
-	while (! (repl = e.next()).empty() )
-	  add_nearmiss(repl.misspelled_word, ms, 
-		       dont_count, dont_need_alloc, repl.elements);	  
       }
     }
   }
@@ -277,125 +310,239 @@ namespace aspeller_default_suggest {
   //
   // try_others - tries to come up with possible suggestions
   //
-  
+
   void Working::try_others () {
 
+    try_split();
+
+    if (parms.soundslike_level == 1 && !fast_scan)
+      try_one_edit();
+    else
+      try_scan();
+    
+    if (!use_soundslike && parms.use_repl_table)
+      try_repl();
+
+    //try_ngram();
+
+  }
+
+  void Working::try_split() {
     const String & word       = original_word.word;
-    const String & soundslike = original_word.soundslike;
     
-    String::size_type i;
+    if (word.size() < 4 || parms.split_chars.empty()) return;
+    size_t i = 0;
     
-    String new_soundslike;
-    new_soundslike.reserve(soundslike.size() + 1);
-
-    // Insert a space or hyphone
-
-    if (word.size() >= 4) {
-
-      char * new_word = new char[word.size() + 2];
-      strncpy(new_word, word.data(), word.size());
-      new_word[word.size() + 1] = '\0';
-      new_word[word.size() + 0] = new_word[word.size() - 1];
-
-      for (i = word.size() - 2; i >= 2; --i) {
-	new_word[i+1] = new_word[i];
-	new_word[i] = '\0';
-	
-	if (speller->check(new_word) && speller->check(new_word + i + 1)) {
-	  new_word[i] = ' ';
-	  add_nearmiss(new_word, parms.edit_distance_weights.del2,
-		       dont_count, do_need_alloc);
-
-	  new_word[i] = '-';
-	  add_nearmiss(new_word, parms.edit_distance_weights.del2,
-		       dont_count, do_need_alloc);
-	}
+    char * new_word = new char[word.size() + 2];
+    strncpy(new_word, word.data(), word.size());
+    new_word[word.size() + 1] = '\0';
+    new_word[word.size() + 0] = new_word[word.size() - 1];
+    
+    for (i = word.size() - 2; i >= 2; --i) {
+      new_word[i+1] = new_word[i];
+      new_word[i] = '\0';
+      
+      if (speller->check(new_word) && speller->check(new_word + i + 1)) {
+        for (size_t j = 0; j != parms.split_chars.size(); ++j)
+        {
+          new_word[i] = parms.split_chars[j];
+          add_nearmiss(new_word, parms.edit_distance_weights.del2*3/2,
+                       dont_count, do_need_alloc);
+        }
       }
-      
-      delete[] new_word;
     }
-    const char * original_soundslike = original_word.soundslike.c_str();
-    //unsigned int original_soundslike_len = strlen(original_soundslike);
-      
-    EditDist (* edit_dist_fun)(const char *, const char *, 
-			       const EditDistanceWeights &);
+    
+    delete[] new_word;
+  }
 
+  void Working::try_one_edit() 
+  {
+    const String & soundslike = active_soundslike();
+    const char * replace_list = soundslike_chars();
+    char a,b;
+    const char * c;
+    String new_soundslike;
+    size_t i;
+
+    // First try the soundslike as is
+
+    try_sound(soundslike, 0);
+
+    // Change one letter
+    
+    new_soundslike = soundslike;
+    
+    for (i = 0; i != soundslike.size(); ++i) {
+      for (c = replace_list; *c; ++c) {
+        if (*c == soundslike[i]) continue;
+        new_soundslike[i] = *c;
+        try_sound(new_soundslike, parms.edit_distance_weights.sub);
+      }
+      new_soundslike[i] = soundslike[i];
+    }
+    
+    // Interchange two adjacent letters.
+    
+    for (i = 0; i+1 != soundslike.size(); ++i) {
+      a = new_soundslike[i];
+      b = new_soundslike[i+1];
+      new_soundslike[i] = b;
+      new_soundslike[i+1] = a;
+      try_sound(new_soundslike,parms.edit_distance_weights.swap);
+      new_soundslike[i] = a;
+      new_soundslike[i+1] = b;
+    }
+
+    // Add one letter
+
+    new_soundslike += ' ';
+    i = new_soundslike.size()-1;
+    while(true) {
+      for (c=replace_list; *c; ++c) {
+        new_soundslike[i] = *c;
+        try_sound(new_soundslike,parms.edit_distance_weights.del1);
+      }
+      if (i == 0) break;
+      new_soundslike[i] = new_soundslike[i-1];
+      --i;
+    }
+    
+    // Delete one letter
+
+    if (soundslike.size() > 1) {
+      new_soundslike = soundslike;
+      a = new_soundslike[new_soundslike.size() - 1];
+      new_soundslike.resize(new_soundslike.size() - 1);
+      i = new_soundslike.size();
+      while (true) {
+        try_sound(new_soundslike,parms.edit_distance_weights.del2);
+        if (i == 0) break;
+        b = a;
+        a = new_soundslike[i-1];
+        new_soundslike[i-1] = b;
+        --i;
+      }
+    }
+  }
+
+  void Working::try_scan() 
+  {
+    const char * original_soundslike = active_soundslike().c_str();
+    //unsigned int original_soundslike_len = strlen(original_soundslike);
+    
+    EditDist (* edit_dist_fun)(const char *, const char *, 
+                               const EditDistanceWeights &);
+    
     if (parms.soundslike_level == 1)
       edit_dist_fun = limit1_edit_distance;
     else
       edit_dist_fun = limit2_edit_distance;
 
-    for (SpellerImpl::DataSetCollection::const_iterator i 
-	   = speller->data_set_collection().begin();
-	 i != speller->data_set_collection().end();
-	 ++i) {
-
-      if (!i->use_to_suggest) continue;
+    WordEntry * sw;
+    WordEntry w;
+    const char * sl = 0;
+    String sl_buf;
+    String word;
+    EditDist score;
+    unsigned int stopped_at = LARGE_NUM;
+    CheckList cl;
+    const CheckInfo * ci_cur;
+    
+    for (SpellerImpl::WS::const_iterator i = speller->suggest_ws.begin();
+         i != speller->suggest_ws.end();
+         ++i) 
+    {
+      StackPtr<SoundslikeEnumeration> els(i->ws->soundslike_elements());
       
-      if (i->data_set->basic_type == DataSet::basic_word_set) {
+      while ( (sw = els->next(stopped_at)) ) {
 
-	const BasicWordSet * data_set 
-	  = static_cast<const BasicWordSet *>(i->data_set);
+        if (sw->what != WordEntry::Word) {
+          sl = sw->word;
+        } else if (!*sw->aff) {
+          sl_buf.clear();
+          to_stripped(*lang, sw->word, sl_buf);
+          sl = sl_buf.c_str();
+        } else {
+          goto affix_case;
+        }
 
-	StackPtr<SoundslikeEnumeration> els(data_set->soundslike_elements());
-    
-	SoundslikeWord sw;	
-	EditDist score;
-        unsigned int stopped_at = LARGE_NUM;
-        //CERR << "\\" << original_soundslike << '\n';
-	while ( (sw = els->next(stopped_at)) == true) 
-	{
-	  score = edit_dist_fun(sw.soundslike,
-				original_soundslike, 
-				parms.edit_distance_weights);
-	  stopped_at = score.stopped_at - sw.soundslike;
-	  if (score < LARGE_NUM) {
-            //CERR << "//" << sw.soundslike << ' ' << score << '\n';
-	    stopped_at = LARGE_NUM;
-	    BasicWordSet::Emul e = data_set->words_w_soundslike(sw);
-	    BasicWordInfo bw;
-	    String word;
-	    while ((bw = e.next())) {
-	      bw.get_word(word, i->local_info.convert);
-	      add_nearmiss(word.c_str(), score, do_count, do_need_alloc);
-	    }
-	  }
-	}
+        score = edit_dist_fun(sl, original_soundslike, parms.edit_distance_weights);
+        stopped_at = score.stopped_at - sl;
+        if (score >= LARGE_NUM) continue;
+        stopped_at = LARGE_NUM;
+        i->ws->soundslike_lookup(*sw, w);
+	//CERR << sw->word << "\n";
+        for (; !w.at_end(); w.adv()) {
+	  //CERR << "  " << w.word << "\n";
+          word.clear();
+          i->convert.convert(w.word, word);
+          WordEntry * repl = 0;
+          if (w.what == WordEntry::Misspelled) {
+            repl = new WordEntry;
+            const BasicReplacementSet * repl_set
+              = static_cast<const BasicReplacementSet *>(i->ws);
+            repl_set->repl_lookup(w, *repl);
+          }
+          add_nearmiss(word.c_str(), score, do_count, do_need_alloc, repl);
+        }
+        continue;
 
-      } else {
-	
-	const BasicReplacementSet * repl_set
-	  = static_cast<const BasicReplacementSet *>(i->data_set);
+      affix_case:
 
-	StackPtr<SoundslikeEnumeration> els(repl_set->soundslike_elements());
-    
-	SoundslikeWord w;
-	EditDist score;
-        unsigned int stopped_at = LARGE_NUM;
-	while ( (w = els->next(stopped_at)) == true) 
-	{
-	  score = edit_dist_fun(w.soundslike,
-				original_soundslike, 
-				parms.edit_distance_weights);
-	  
-	  stopped_at = score.stopped_at - w.soundslike;
-	  if (score < LARGE_NUM) {
-	    stopped_at = LARGE_NUM;
-	    BasicReplacementSet::Emul e = repl_set->repls_w_soundslike(w);
-	    ReplacementList repl;
-	    while (! (repl = e.next()).empty() )
-	      add_nearmiss(repl.misspelled_word, score, 
-			   dont_count, dont_need_alloc, repl.elements);
-	  }
-	}
+        lang->affix()->expand(sw->word, sw->aff, &cl);
+        ci_cur = cl.data + 1;
+        for (;ci_cur; ci_cur = ci_cur->next) {
+          sl_buf.clear();
+          to_stripped(*lang, ci_cur->word, sl_buf);
+          score = edit_dist_fun(sl_buf.c_str(), original_soundslike, parms.edit_distance_weights);
+          stopped_at = score.stopped_at - sl;
+          if (score >= LARGE_NUM) continue;
+          stopped_at = LARGE_NUM;
+          word.clear();
+          i->convert.convert(ci_cur->word, word);
+          add_nearmiss(word.c_str(), score, do_count, do_need_alloc, 0);
+        }
       }
     }
   }
 
+  struct ReplTry 
+  {
+    const char * begin;
+    const char * end;
+    const char * repl;
+    size_t repl_len;
+    ReplTry(const char * b, const char * e, const char * r)
+      : begin(b), end(e), repl(r), repl_len(strlen(r)) {}
+  };
+
+  // only use when soundslike == stripped
+  void Working::try_repl() 
+  {
+    CharVector buf;
+    Vector<ReplTry> repl_try;
+    StackPtr<SuggestReplEnumeration> els(lang->repl());
+    const SuggestRepl * r = 0;
+    const char * word = original_word.stripped.c_str();
+    const char * wend = word + original_word.stripped.size();
+    while (r = els->next(), r) 
+    {
+      const char * p = word;
+      while ((p = strstr(p, r->substr))) {
+        buf.clear();
+        buf.append(word, p);
+        buf.append(r->repl, strlen(r->repl));
+        p += strlen(r->substr);
+        buf.append(p, wend + 1);
+        try_sound(buf.data(), parms.edit_distance_weights.sub*3/2);
+      }
+    }
+  }
+
+  
   void Working::score_list() {
     if (near_misses.empty()) return;
 
-    bool no_soundslike = strcmp(speller->lang().soundslike_name(), "none") == 0;
     parms.set_original_word_size(original_word.word.size());
       
     NearMisses::iterator i;
@@ -423,10 +570,10 @@ namespace aspeller_default_suggest {
 
 	int level = needed_level(try_for, i->soundslike_score);
 	
-	if (no_soundslike)
+	if (!use_soundslike)
 	  word_score = i->soundslike_score;
 	else if (level >= int(i->soundslike_score/parms.edit_distance_weights.min))
-	  word_score = edit_distance(original_word.word_stripped.c_str(),
+	  word_score = edit_distance(original_word.stripped.c_str(),
 				     i->word_stripped,
 				     level, level,
 				     parms.edit_distance_weights);
@@ -484,10 +631,10 @@ namespace aspeller_default_suggest {
       int initial_level = needed_level(try_for, i->soundslike_score);
       int max_level = needed_level(threshold, i->soundslike_score);
 	
-      if (no_soundslike)
+      if (!use_soundslike)
 	word_score = i->soundslike_score;
       else if (initial_level < max_level)
-	word_score = edit_distance(original_word.word_stripped.c_str(),
+	word_score = edit_distance(original_word.stripped.c_str(),
 				   i->word_stripped,
 				   initial_level+1,max_level,
 				   parms.edit_distance_weights);
@@ -520,7 +667,7 @@ namespace aspeller_default_suggest {
       CharVector original, word;
       original.resize(original_word.word.size() + 1);
       for (j = 0; j != original_word.word.size(); ++j)
-        original[j] = lang->to_normalized(original_word.word[j]);
+          original[j] = lang->to_normalized(original_word.word[j]);
       original[j] = 0;
       word.resize(max_word_length + 1);
       
@@ -566,17 +713,16 @@ namespace aspeller_default_suggest {
            << '\t' << lang->to_soundslike(i->word) << "\n";
 #    endif
       if (i->repl_list != 0) {
-	const char * word;
-	string::size_type pos;
-	while((word = i->repl_list->next()) != 0) {
-	  dup_pair = duplicates_check.insert(fix_case(word));
-	  if (dup_pair.second && 
-	      ((pos = dup_pair.first->find(' '), pos == String::npos)
-	       ? (bool)speller->check(*dup_pair.first)
-	       : (speller->check((String)dup_pair.first->substr(0,pos)) 
-		  && speller->check((String)dup_pair.first->substr(pos+1))) ))
-	    near_misses_final->push_back(*dup_pair.first);
-	}
+ 	string::size_type pos;
+	do {
+ 	  dup_pair = duplicates_check.insert(fix_case(i->repl_list->word));
+ 	  if (dup_pair.second && 
+ 	      ((pos = dup_pair.first->find(' '), pos == String::npos)
+ 	       ? (bool)speller->check(*dup_pair.first)
+ 	       : (speller->check((String)dup_pair.first->substr(0,pos)) 
+ 		  && speller->check((String)dup_pair.first->substr(pos+1))) ))
+ 	    near_misses_final->push_back(*dup_pair.first);
+ 	} while (i->repl_list->adv());
       } else {
 	dup_pair = duplicates_check.insert(fix_case(i->word));
 	if (dup_pair.second )
@@ -585,14 +731,6 @@ namespace aspeller_default_suggest {
     }
   }
   
-  void Working::get_suggestions(NearMissesFinal & sug) {
-    near_misses_final = & sug;
-    if (original_word.soundslike.empty()) return;
-    try_others();
-    score_list();
-    transfer();
-  }
-  
   class SuggestionListImpl : public SuggestionList {
     struct Parms {
       typedef const char *                    Value;
@@ -614,7 +752,7 @@ namespace aspeller_default_suggest {
     bool empty() const { return suggestions.empty(); }
     Size size() const { return suggestions.size(); }
     VirEmul * elements() const {
-      return new MakeVirEnumeration<Parms, StringEnumeration>
+      return new MakeEnumeration<Parms, StringEnumeration>
 	(suggestions.begin(), Parms(suggestions.end()));
     }
   };
@@ -624,12 +762,10 @@ namespace aspeller_default_suggest {
     SuggestionListImpl  suggestion_list;
     SuggestParms parms_;
   public:
-    SuggestImpl(SpellerImpl * m) 
-      : speller_(m), parms_(m->config()->retrieve("sug-mode")) 
-    {parms_.fill_distance_lookup(m->config(), m->lang());}
-    SuggestImpl(SpellerImpl * m, const SuggestParms & p) 
-      : speller_(m), parms_(p) 
-    {parms_.fill_distance_lookup(m->config(), m->lang());}
+    SuggestImpl(SpellerImpl * m);
+    //SuggestImpl(SpellerImpl * m, const SuggestParms & p)
+    //  : speller_(m), parms_(p) 
+    //{parms_.fill_distance_lookup(m->config(), m->lang());}
     PosibErr<void> set_mode(ParmString mode) {
       return parms_.set(mode);
     }
@@ -644,6 +780,24 @@ namespace aspeller_default_suggest {
     }
     SuggestionList & suggest(const char * word);
   };
+  
+  SuggestImpl::SuggestImpl(SpellerImpl * m)
+    : speller_(m), parms_(m->config()->retrieve("sug-mode")) 
+  {
+    if (m->config()->retrieve("sug-mode") == "normal" 
+        && !m->fast_scan) parms_.soundslike_level = 1;
+
+    if (m->config()->have("sug-edit-dist"))
+      parms_.soundslike_level = m->config()->retrieve_int("sug-edit-dist"); // FIXME: Make sure 1 or 2
+    if (m->config()->have("sug-typo-analysis"))
+      parms_.use_typo_analysis = m->config()->retrieve_bool("sug-typo-analysis");
+    if (m->config()->have("sug-repl-table"))
+      parms_.use_repl_table = m->config()->retrieve_bool("sug-repl-table");
+    
+    parms_.split_chars = m->config()->retrieve("sug-split-chars");
+
+    parms_.fill_distance_lookup(m->config(), m->lang());
+  }
 
   SuggestionList & SuggestImpl::suggest(const char * word) { 
 #   ifdef DEBUG_SUGGEST
@@ -666,15 +820,12 @@ namespace aspeller {
     return new aspeller_default_suggest::SuggestImpl(m);
   }
 
-  Suggest * new_default_suggest(SpellerImpl * m, const SuggestParms & p) {
-    return new aspeller_default_suggest::SuggestImpl(m,p);
-  }
+  //Suggest * new_default_suggest(SpellerImpl * m, const SuggestParms & p) {
+  //  return new aspeller_default_suggest::SuggestImpl(m,p);
+  //}
 
   PosibErr<void> SuggestParms::set(ParmString mode) {
 
-    if (mode != "normal" && mode != "fast" && mode != "ultra" && mode != "bad-spellers")
-      return make_err(bad_value, "sug-mode", mode, "one of ultra, fast, normal, or bad-spellers");
-
     edit_distance_weights.del1 =  95;
     edit_distance_weights.del2 =  95;
     edit_distance_weights.swap =  90;
@@ -689,36 +840,50 @@ namespace aspeller {
 
     soundslike_weight = normal_soundslike_weight;
     word_weight       = 100 - normal_soundslike_weight;
-      
+
+    split_chars = " -";
+
     skip = 2;
     limit = 100;
     if (mode == "normal") {
       use_typo_analysis = true;
+      use_repl_table = true;
+      soundslike_level = 2; // either one or two
+      span = 50;
+    } else if (mode == "slow") {
+      use_typo_analysis = true;
+      use_repl_table = true;
       soundslike_level = 2; // either one or two
       span = 50;
     } else if (mode == "fast") {
       use_typo_analysis = true;
+      use_repl_table = true;
       soundslike_level = 1; // either one or two
       span = 50;
     } else if (mode == "ultra") {
       use_typo_analysis = false;
+      use_repl_table = false;
       soundslike_level = 1; // either one or two
       span = 50;
     } else if (mode == "bad-spellers") {
       use_typo_analysis = false;
+      use_repl_table = true;
       normal_soundslike_weight = 55;
       small_word_threshold = 0;
       soundslike_level = 2; // either one or two
       span = 125;
       limit = 1000;
     } else {
-      abort(); // this should NEVER happen.
+      return make_err(bad_value, "sug-mode", mode, "one of ultra, fast, normal, slow, or bad-spellers");
     }
 
     return no_err;
   }
 
   PosibErr<void> SuggestParms::fill_distance_lookup(const Config * c, const Language & l) {
+
+    // FIXME: avoid having to recreate the tables each time
+    //        instead, cache the table and use the cached copy
     
     TypoEditDistanceWeights & w = typo_edit_distance_weights;
 
@@ -747,18 +912,19 @@ namespace aspeller {
 	}
       }
 
-      String key, data;
-      while (getdata_pair(in, key, data)) {
-	if (key.size() != 2) 
+      FixedBuffer<64> buf;
+      DataPair d;
+      while (getdata_pair(in, d, buf)) {
+	if (d.key.size() != 2)
 	  return make_err(bad_file_format, file);
-	w.repl (l.to_normalized(key[0]),
-		l.to_normalized(key[1])) = w.repl_dis1;
-	w.repl (l.to_normalized(key[1]),
-		l.to_normalized(key[0])) = w.repl_dis1;
-	w.extra(l.to_normalized(key[0]),
-		l.to_normalized(key[1])) = w.extra_dis1;
-	w.extra(l.to_normalized(key[1]),
-		l.to_normalized(key[0])) = w.extra_dis1;
+	w.repl (l.to_normalized(d.key[0]),
+		l.to_normalized(d.key[1])) = w.repl_dis1;
+	w.repl (l.to_normalized(d.key[1]),
+		l.to_normalized(d.key[0])) = w.repl_dis1;
+	w.extra(l.to_normalized(d.key[0]),
+		l.to_normalized(d.key[1])) = w.extra_dis1;
+	w.extra(l.to_normalized(d.key[1]),
+		l.to_normalized(d.key[0])) = w.extra_dis1;
       }
 
       for (int i = 0; i != c; ++i) {
